//
// PS2PC: An Intel x86 emulator for MIPS R5900 (Sony PS2).
// Coded by Mike Tedder / breakpoint - bpoint@parodius.com
// http://ps2pc.parodius.com
//
// This project is released as 'open source'.  You may make any modifications
// or adjustments to the code and submit a patch to be applied to the project.
// See the web page for more details.
//
//-----------------------------------------------------------------------

#include "config.h"
#include "types.h"
#include "dynrec.h"
#include "cpu.h"
#include "x86.h"
#include "debug.h"

#include <stdlib.h>				// for malloc() and free()


#if (!CFG_DYNREC_CACHE_USE_TLB)
 typedef struct
 {
	ulong  pcPhysAddr;				// physical address in x86-land
	void  *compiledAddr;			// pointer to pre-compiled code
	ulong  size;					// length of compiled code
	ulong  timesUsed;				// number of times this block of pre-compiled code has been used
 } dynrecCacheEntry;
#endif

void *drCachePool = NULL;			// pointer to dynrec cache pool
#if (!CFG_DYNREC_CACHE_USE_TLB)
 dynrecCacheEntry *drCacheEntryPool = NULL;		// pointer to dynrec cache entry pool
#endif
void *gdrCompileBuffer = NULL;		// pointer to temporary compile buffer (where all output code goes)


bool dynrecInit(void)
{
	// Check we're not already inited...
	if (drCachePool != NULL)
	{
		debugPrintf("dynrec core already inited!\n");
		return FALSE;
	}

	// Allocate cache pool...
	drCachePool = malloc(CFG_DYNREC_CACHE_SIZE * 1024);
	if (drCachePool == NULL) return FALSE;

#if (!CFG_DYNREC_CACHE_USE_TLB)
	// Allocate cache entry pool...
	drCacheEntryPool = (dynrecCacheEntry *)malloc(CFG_DYNREC_CACHE_ENTRIES * sizeof(dynrecCacheEntry));
	if (drCacheEntryPool == NULL) return FALSE;
#endif

	// Allocate compile buffer...
	gdrCompileBuffer = malloc(CFG_DYNREC_COMPILE_BUF_SIZE * 1024);
	if (gdrCompileBuffer == NULL) return FALSE;

	return TRUE;
}

void dynrecShutdown(void)
{
	// Make sure we're not already shutdown...
	if (drCachePool == NULL)
	{
		debugPrintf("dynrec core already shutdown!\n");
		return;
	}

	free(drCachePool);
	drCachePool = NULL;

#if (!CFG_DYNREC_CACHE_USE_TLB)
	// Free cache entry pool...
	free(drCacheEntryPool);
	drCacheEntryPool = NULL;
#endif

	// Free compile buffer...
	free(gdrCompileBuffer);
	gdrCompileBuffer = NULL;
}


int dynrecCompileBlock(uchar *in, ulong *out, int *outLen)
{
	bool isBranch = FALSE;
	uchar *initialIn = in;
	ulong *initialOut = out, skip, size;

	while (isBranch == FALSE)
	{
		skip = 0;
#if (!DO_DEBUG)
		in  += x86OpcodeTable[(uchar)*in](in, out, &isBranch, &skip);
#else
		if ((size = x86OpcodeTable[(uchar)*in](in, out, &isBranch, &skip)) == 0)
		{
			debugPrintf("'Not done' opcode encountered in block around CS:EIP= %04X:%08X\n", gCpu.cs, gCpu.eip);
			debugPrintf("16 byte dump of current opcode to translate:\n");
			debugPrintf("    %02X %02X %02X %02X %02X %02X %02X %02X  %02X %02X %02X %02X %02X %02X %02X %02X\n",
				*(in    ), *(in + 1), *(in +  2), *(in +  3), *(in +  4), *(in +  5), *(in +  6), *(in +  7),
				*(in + 8), *(in + 9), *(in + 10), *(in + 11), *(in + 12), *(in + 13), *(in + 14), *(in + 15));

			isBranch = TRUE;
			return -1;
		}
		else
		{
			in += size;
		}
#endif

		out += skip;

		// Check to see if it's time to clear the opSize prefix...
		if (gOpSizeCounter)
		{
			if ((--gOpSizeCounter) == 0)
			{
				if (gIn32BitMode)
					gIsOpSize32 = TRUE;
				else
					gIsOpSize32 = FALSE;
			}
		}

		// Check to see if it's time to clear the addrSize prefix...
		if (gAddrSizeCounter)
		{
			if ((--gAddrSizeCounter) == 0)
			{
				if (gIn32BitMode)
					gIsAddrSize32 = TRUE;
				else
					gIsAddrSize32 = FALSE;
			}
		}

		// Check to see if it's time to clear the segOvr prefix...
		if (gSegOvrCounter)
		{
			if ((--gSegOvrCounter) == 0)
				gIsSegOverride = FALSE;
		}
	}

	*outLen = (out - initialOut) << 2;
	return (in - initialIn);
}
