//
// PS2PC: An Intel x86 emulator for MIPS R5900 (Sony PS2).
// Coded by Mike Tedder / breakpoint - bpoint@parodius.com
// http://ps2pc.parodius.com
//
// This project is released as 'open source'.  You may make any modifications
// or adjustments to the code and submit a patch to be applied to the project.
// See the web page for more details.
//
//-----------------------------------------------------------------------

#include "config.h"
#include "types.h"
#if ((PC_VERSION && DEVICE_SUPPORT) || (PS2_VERSION))
 #include "devices.h"
#endif
#include "mem.h"
#include "debug.h"
#include "dynrec.h"
#include "cpu.h"

#include <stdio.h>


#if (PS2_VERSION)
//==================================================================================================================
// PS2 main entry...
int main(void)
{
	FILE *fp;
	void *biosAddr;
	bool done = FALSE;

#if (!SKY_VERSION)
	// initialize ps2 hardware...
	sceDevVif0Reset();
#endif

	// initialize debug system...
	debugInit();

	// initialize dynrec compiler cache space...
	if (dynrecInit() == FALSE)
	{
		debugPrintf("out of RAM allocating dynrec cache!\n");
		return 1;
	}

	debugPrintf("Dynrec init success.\n");

	// initialize memory system...
	if (memInit() == FALSE)
	{
		debugPrintf("out of RAM allocating PC memory!\n");
		return 1;
	}

	debugPrintf("Memory init success -- %dMb available, base RAM pointer @ 0x%08X.\n", CFG_X86_MEMORY_SIZE, memGetBase());

	// initialize device system...
	devInit();

	// load BIOS into memory...
	biosAddr = memCalcSegOfsAddr(0xF000, 0x0000);
	debugPrintf("Loading BIOS at F000:0000 (addr=0x%08X)...\n", biosAddr);

	fp = fopen("bios.bin", "rb");
	if (fp == NULL)
	{
		debugPrintf("Can't open BIOS: bios.bin!\n");
		return 1;
	}

	// read in BIOS...
	fread(biosAddr, 1, 65536, fp);
	fclose(fp);

	// reset x86 CPU...
	cpuReset();

	// begin executing code!
	done = FALSE;
	while (!done)
	{
		if (cpuExec() == FALSE) done = TRUE;

		// TODO: maybe handle special pad routines here to control emulation (and exit (?))....
	}

	// shutdown memory system...
	memShutdown();

	// shutdown dynrec compiler...
	dynrecShutdown();

	// shutdown debug system...
	debugShutdown();
	return 0;
}
#endif // (PS2_VERSION)


#if (PC_VERSION)
//==================================================================================================================
// PC main entry...
#include <stdlib.h>
#include <ctype.h>
#include <memory.h>

#if (DEVICE_SUPPORT)
void scanDeviceCallback(int devIdx, devDeviceDefinition *devBlk)
{
	uint t;

	printf("PS2PC: Device #%d (%s)", devIdx, devBlk->devName);

	if (devBlk->devFlags & DEVFLAGS_USES_IO)
	{
		printf(" - Ports: ");

		for (t = 0; t < devBlk->devPortCount; t++)
		{
			if (devBlk->devBeginPortList[t] == devBlk->devEndPortList[t])
				printf("%X, ", devBlk->devBeginPortList[t]);
			else
				printf("%X-%X, ", devBlk->devBeginPortList[t], devBlk->devEndPortList[t]);
		}

		printf("\b\b \b");
	}

	if (devBlk->devFlags & DEVFLAGS_USES_MEMORY)
	{
		printf(" - Memory: ");

		for (t = 0; t < devBlk->devMemoryCount; t++)
		{
			if (devBlk->devBeginMemoryList[t] == devBlk->devEndMemoryList[t])
				printf("%08X, ", devBlk->devBeginMemoryList[t]);
			else
				printf("%08X-%08X, ", devBlk->devBeginMemoryList[t], devBlk->devEndMemoryList[t]);
		}

		printf("\b\b \b");
	}

	printf("\n");
}
#endif

void main(int argc, char *argv[])
{
	char *infile, *outfile;
	ulong *outbuf;
	FILE *fp;
	int size, xlated, t;
	bool bigEndian;
	ulong *cptr, temp;
	void *inAddr;

/*
// segmented mode addressing test -- DOES NOT COMPILE WITH MSVC6 (due to lack of 16-bit support)
	__asm {
		mov ax, 0x1000
		mov ds, ax
		mov bx, 0xFFFF
		mov si, 0x0001
		mov al, [bx+si]
	};
*/

/*
// aux carry test (flag set when bit 4 of an instruction is changed [either set or reset, depending on last state])
	__asm {
		mov ebx, 0x7FFDF00F
		inc ebx
		inc ebx
		mov ebx, 0x7FFDF01F
		inc ebx
		dec ebx
		dec ebx
		inc ebx
		inc ebx
	};
*/

	printf("PS2PC: An Intel x86 emulator for MIPS R5900 (Sony PS2).\n");
	printf("Coded by Mike Tedder / breakpoint - http://ps2pc.parodius.com\n\n");

	if (argc < 3)
	{
		printf("Usage: PS2PC [-e] [-3] <infile> <outfile>\n\n");
		printf("Where <infile> is a file containing a x86 code fragment and <outfile> is a\n");
		printf("file to write the resultant R5900 code to.\n\n");
		printf("Options:\n");
		printf("    -e: write out a big endian version of the code (for NIEW64)\n");
		printf("    -3: assume 32-bit mode (EAX, EBX, etc. support)\n");
		printf("\n");
		return;
	}

	// Init vars.
	infile    = NULL;
	outfile   = NULL;
	bigEndian = FALSE;

	// Parse command line...
	printf("PS2PC: Parsing command line...\n");

	for (t = 1; t < argc; t++)
	{
		if (argv[t][0] == '-')
		{
			switch (toupper(argv[t][1]))
			{
				case 'E':
					printf("PS2PC: Big-endian format enabled.\n");
					bigEndian = TRUE;
					break;

				case '3':
					printf("PS2PC: 32-bit mode enabled.\n");
					gIn32BitMode = TRUE;
					break;

				default:
					printf("PS2PC: Unknown option '%s'!\n", argv[t]);
					return;
			}
		}
		else
		{
			// Not an option, so assume infile/outfile...
			if (infile == NULL)
				infile = argv[t];
			else
			{
				if (outfile == NULL)
					outfile = argv[t];
				else
				{
					printf("PS2PC: Unknown argument '%s'!\n", argv[t]);
					return;
				}
			}
		}
	}

#if (DEVICE_SUPPORT)
	printf("PS2PC: Initializing devices...\n");
	devInit();

	printf("PS2PC: Scanning device list...\n");
	devScanDevices(scanDeviceCallback);

	printf("PS2PC: Calling init on device 0...\n");
	devInitDevice(0);
#endif

	printf("PS2PC: Debug init...\n");
	debugInit();

	printf("PS2PC: Dynrec init...\n");
	if (dynrecInit() == FALSE)
	{
		printf("PS2PC: Dynrec init failure, out of RAM!\n");
		return;
	}

	printf("PS2PC: Memory init (allocing %dMb of RAM)...\n", CFG_X86_MEMORY_SIZE);
	if (memInit() == FALSE)
	{
		printf("PS2PC: Memory init failure, out of RAM!\n");
		return;
	}

	printf("PS2PC: Memory init success, base RAM pointer @ 0x%08X.\n", memGetBase());

	inAddr = memCalcSegOfsAddr(CFG_CODE_LOAD_SEG, CFG_CODE_LOAD_OFS);
	printf("PS2PC: Loading code from %s, mapping at %04X:%04X (addr=0x%08X)...\n", infile, CFG_CODE_LOAD_SEG, CFG_CODE_LOAD_OFS, inAddr);

	fp = fopen(infile, "rb");
	if (fp == NULL)
	{
		printf("PS2PC: Can't open input code!\n");
		return;
	}

	// Seek to eof, get code size.
	fseek(fp, 0, 2);
	size = ftell(fp);
	rewind(fp);

	// Read in code...
	fread(inAddr, 1, size, fp);
	fclose(fp);

	// Alloc space for output code.
	outbuf = (ulong *)malloc(size * 32);
	if (outbuf == NULL)
	{
		printf("PS2PC: Can't alloc space for output code!\n");
		return;
	}

	printf("PS2PC: Compiling code...\n");

	xlated = dynrecCompileBlock((uchar *)inAddr, outbuf, &size);

	printf("PS2PC: Hit a branch, done compiling.\n");
	printf("PS2PC: Xlated %d bytes of x86 opcodes, resultant R5900 size is %d bytes\n", xlated, size);

	// Flip endian, if requested...
	if (bigEndian)
	{
		printf("PS2PC: Converting to big endian output...\n");

		cptr = (ulong *)outbuf;
		for (t = 0; t < (size >> 2); t++)
		{
			 temp = *cptr;
			*cptr = ((temp & 0xFF000000) >> 24) | ((temp & 0x00FF0000) >> 8) | ((temp & 0x0000FF00) << 8) | ((temp & 0x000000FF) << 24);
			 cptr++;
		}
	}

	printf("PS2PC: Writing output code to %s...\n", outfile);

	fp = fopen(outfile, "wb");
	fwrite(outbuf, 1, size, fp);
	fclose(fp);

	printf("PS2PC: Memory shutdown...\n");
	memShutdown();

	printf("PS2PC: Dynrec shutdown...\n");
	dynrecShutdown();

	printf("PS2PC: Debug shutdown...\n");
	debugShutdown();

	printf("PS2PC: Done!\n");
}
#endif // (PC_VERSION)
