//
// PS2PC: An Intel x86 emulator for MIPS R5900 (Sony PS2).
// Coded by Mike Tedder / breakpoint - bpoint@parodius.com
// http://ps2pc.parodius.com
//
// This project is released as 'open source'.  You may make any modifications
// or adjustments to the code and submit a patch to be applied to the project.
// See the web page for more details.
//
//-----------------------------------------------------------------------

#include "config.h"
#include "types.h"
#include "x86.h"
#include "x86mov.h"
#include "r59basic.h"
#include "mem.h"


//=====================
//  MOV Helper Funcs
//=====================

int x86MoveReg8LReg8L(ulong *out, x86RegByte xSrcReg, x86RegByte xDestReg)
{
	uchar rSrc  = x86RegByteToR5900Reg[xSrcReg];
	uchar rDest = x86RegByteToR5900Reg[xDestReg];

	* out      = r5900ANDI (rAT, rSrc, 0x00FF);	// temp1 = 0x000000ss (s = src)
	*(out + 1) = r5900ADDIU(rS0, rR0, 0xFF00);	// temp2 = 0xFFFFFF00
	*(out + 2) = r5900AND  (rS1, rDest, rS0);	// temp3 = 0xdddddd00 (d = dest)
	*(out + 3) = r5900OR   (rDest, rAT, rS1);	// dest  = 0xddddddss
	
	return 4;
}

int x86MoveReg8HReg8H(ulong *out, x86RegByte xSrcReg, x86RegByte xDestReg)
{
	uchar rSrc  = x86RegByteToR5900Reg[xSrcReg];
	uchar rDest = x86RegByteToR5900Reg[xDestReg];

	* out      = r5900ANDI(rAT, rSrc, 0xFF00);	// temp1 = 0x0000ss00 (s = src)
	*(out + 1) = r5900LUI (rS0, 0xFFFF);		// temp2 = 0xFFFF0000
	*(out + 2) = r5900ORI (rS1, rS0, 0x00FF);	// temp3 = 0xFFFF00FF
	*(out + 3) = r5900AND (rS0, rDest, rS1);	// temp2 = 0xdddd00dd (d = dest)
	*(out + 4) = r5900OR  (rDest, rAT, rS0);	// dest  = 0xddddssdd

	return 5;
}

int x86MoveReg8LReg8H(ulong *out, x86RegByte xSrcReg, x86RegByte xDestReg)
{
	uchar rSrc  = x86RegByteToR5900Reg[xSrcReg];
	uchar rDest = x86RegByteToR5900Reg[xDestReg];

	* out      = r5900ANDI(rS0, rSrc, 0x00FF);	// temp2 = 0x000000ss (s = src)
	*(out + 1) = r5900SLL (rAT, rS0, 8);		// temp1 = 0x0000ss00
	*(out + 2) = r5900LUI (rS0, 0xFFFF);		// temp2 = 0xFFFF0000
	*(out + 3) = r5900ORI (rS1, rS0, 0x00FF);	// temp3 = 0xFFFF00FF
	*(out + 4) = r5900AND (rS0, rDest, rS1);	// temp2 = 0xdddd00dd (d = dest)
	*(out + 5) = r5900OR  (rDest, rAT, rS0);	// dest  = 0xddddssdd

	return 6;
}

int x86MoveReg8HReg8L(ulong *out, x86RegByte xSrcReg, x86RegByte xDestReg)
{
	uchar rSrc = x86RegByteToR5900Reg[xSrcReg];
	uchar rDest = x86RegByteToR5900Reg[xDestReg];

	* out      = r5900ANDI (rS0, rSrc, 0xFF00);	// temp2 = 0x0000ss00 (s = src)
	*(out + 1) = r5900SRL  (rAT, rS0, 8);		// temp1 = 0x000000ss
	*(out + 2) = r5900ADDIU(rS0, rR0, 0xFF00);	// temp2 = 0xFFFFFF00
	*(out + 3) = r5900AND  (rS1, rDest, rS0);	// temp3 = 0xdddddd00 (d = dest)
	*(out + 4) = r5900OR   (rDest, rAT, rS1);	// dest  = 0xddddddss

	return 5;
}


int x86MoveReg8Reg8(ulong *out, x86RegByte srcReg, x86RegByte destReg)
{
    // optimization check...
	if (srcReg == destReg)
		return 0;

	// both high?
	if (REG_IS_HIGH(srcReg) && REG_IS_HIGH(destReg))
		return x86MoveReg8HReg8H(out, srcReg, destReg);

	// maybe just the source...
	if (REG_IS_HIGH(srcReg))
		return x86MoveReg8HReg8L(out, srcReg, destReg);

	// hmm, maybe just the dest?
	if (REG_IS_HIGH(destReg))
		return x86MoveReg8LReg8H(out, srcReg, destReg);

	// okay, neither are high...
	return x86MoveReg8LReg8L(out, srcReg, destReg);
}


int x86MoveReg16Reg16(ulong *out, x86RegWord16 srcReg, x86RegWord16 destReg)
{
	uchar rSrc  = x86RegWordToR5900Reg[srcReg];
	uchar rDest = x86RegWordToR5900Reg[destReg];

    // optimization check...
	if (srcReg == destReg)
		return 0;

	* out      = r5900ANDI(rAT, rSrc, 0xFFFF);			// temp1 = 0x0000ssss (s = src)
	*(out + 1) = r5900LUI (rS0, 0xFFFF);				// temp2 = 0xFFFF0000
	*(out + 2) = r5900ANDI(rS1, rS0, rDest);			// temp3 = 0xdddd0000 (d = dest)
	*(out + 3) = r5900OR  (rDest, rS0, rS1);			// dest  = 0xddddssss

	return 4;
}


ushort x86DecodeDisplacement16(x86MODRMByte16 destAddr, uchar *in, uint *size)
{
	ushort disp = 0;

	// Check for 8-bit displacement first...
	if (MODRM_HAS_DISPL8(destAddr))
	{
		disp   = (uchar) *(in + 2);
		*size += 1;
	}
	else
	{
		// How about 16-bit displacement?
		if (MODRM_HAS_DISPL16(destAddr))
		{
			disp   = (ushort) MAKE_WORD(*(in + 3), *(in + 2));
			*size += 2;
		}
	}

	return disp;
}


ulong x86DecodeDisplacement32(x86MODRMByte32 destAddr, uchar *in, uint *size)
{
	ulong disp = 0;

	// Check for 8-bit displacement first...
	if (MODRM_HAS_DISPL8(destAddr))
	{
		disp   = (uchar) *(in + 2);
		*size += 1;
	}
	else
	{
		// How about 32-bit displacement?
		if (MODRM_HAS_DISPL32(destAddr))
		{
			disp   = (ulong) MAKE_DWORD(*(in + 5), *(in + 4), *(in + 3), *(in + 2));
			*size += 4;
		}
	}

	return disp;
}


ulong x86DecodeDisplacement32SIB(x86MODRMByte32SIB sibAddr, uchar *in, uint *size)
{
	ulong disp = 0;

	// Check for 8-bit displacement first...
	if (SIB_HAS_DISPL8(sibAddr))
	{
		disp   = (uchar) *(in + 3);
		*size += 1;
	}
	else
	{
		// How about 32-bit displacement?
		if (SIB_HAS_DISPL32(sibAddr))
		{
			disp   = (ulong) MAKE_DWORD(*(in + 6), *(in + 5), *(in + 4), *(in + 3));
			*size += 4;
		}
	}

	return disp;
}


//=====================
//        MOV
//=====================

int x86MOV_EbGb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	x86RegByte srcReg, destReg;
	x86MODRMByte16 destAddr;
	x86MODRMByte32SIB sibAddr;
	ulong disp;
	uint size = 0, ss, index;
	register uchar temp;

	// Decode 'reg' and 'modr/m' fields...
	srcReg   = DECODE_REG_BYTE_FIELD(*(in + 1));
	destAddr = DECODE_MODRM_FIELD(*(in + 1));

	// Check if the modr/m field is a register...
	if (MODRM_IS_REGISTER(destAddr))
	{
		// Do an 8-bit reg->reg move.
		destReg = MODRM_GET_REG_BYTE(destAddr);
		*skip  += x86MoveReg8Reg8(out, srcReg, destReg);
		size    = 2;
	}
	else
	{
		// Not reg->reg, so handle memory.
		if (gIsAddrSize32)
		{
			// Check for SIB byte.
			if (MODRM_HAS_SIB(destAddr))
			{
				// Initialize the size of this opcode (thus far).
				size = 3;

				// Decode SIB info...
				temp    = *(in + 2);
				sibAddr = DECODE_SIB_MODBASE(destAddr, temp);
				ss      = DECODE_SIB_SS(temp);
				index   = DECODE_SIB_INDEX(temp);

				// Get the displacement (if any)...
				disp = x86DecodeDisplacement32SIB(sibAddr, in, &size);

				// Write it!
				*skip += mem32SIBWriteByte(out, srcReg, sibAddr, ss, index, disp);
			}
			else
			{
				// Initialize the size of this opcode (thus far).
				size = 2;

				// Get the displacement (if any)...
				disp = x86DecodeDisplacement32((x86MODRMByte32)destAddr, in, &size);

				// Write it!
				*skip += mem32WriteByte(out, srcReg, (x86MODRMByte32)destAddr, disp);
			}
		}
		else
		{
			// Initialize the size of this opcode (thus far).
			size = 2;

			// Get the displacement (if any)...
			disp = x86DecodeDisplacement16(destAddr, in, &size);

			// Now write the byte!
			*skip += mem16WriteByte(out, srcReg, destAddr, (ushort)disp);
		}
	}

	return size;
}


int x86MOV_EvGv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
/*
	x86RegByte srcReg, destReg;
	x86MODRMByte32 destAddr;
	ushort disp;
	uint size;

	// Decode 'reg' and 'modr/m' fields...
	srcReg   = DECODE_REG_BYTE_FIELD(*(in + 1));
	destAddr = DECODE_MODRM_BYTE_FIELD(*(in + 1));

	// Check if the modr/m field is a register...
	if (MODRM_IS_REGISTER(destAddr))
	{
		// Do an 8-bit reg->reg move.
		destReg = MODRM_GET_REG_BYTE(destAddr);
		*skip  += x86MoveReg8Reg8(out, srcReg, destReg);
		size    = 2;
	}
	else
	{
		// Not a register, so check for displacement first.
		if (MODRM_HAS_DISPL8(destAddr))
		{
			disp   = (uchar) *(in + 2);
			*skip += mem16WriteByte(out, srcReg, destAddr, disp);
			size   = 3;
		}
		else
		{
			// No 8-bit displacement, so how about 16-bit?
			if (MODRM_HAS_DISPL16(destAddr))
			{
				disp   = (ushort) MAKE_WORD(*(in + 2), *(in + 3));
				*skip += mem16WriteByte(out, srcReg, destAddr, disp);
				size   = 4;
			}
			else
			{
				// No displacement at all...
				*skip += mem16WriteByte(out, srcReg, destAddr, 0);
				size   = 2;
			}
		}
	}

	return size;

	if (in32BitMode)
	{
		srcReg   = x86DecodeRegWord32Field(*(in + 1));
		destAddr = x86DecodeMODRMWord32Field(*(in + 1));
	}
	else
	{
		srcReg   = x86DecodeRegWord16Field(*(in + 1));
		destAddr = x86DecodeMODRMWord16Field(*(in + 1));
	}

*/
 	return 0;
}


int x86MOV_GbEb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86MOV_GvEv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86MOV_EwSw(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86MOV_SwEw(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86MOV_ALOb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86MOV_eAXOv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86MOV_ObAL(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86MOV_OveAX(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86MOV_ALIb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	uchar val;

	// Get immediate byte value.
	val = *(in + 1);

	// Encode it...
	* out      = r5900ADDIU(rS0, rR0, 0xFF00);				// S0   = 0xFFFFFF00
	*(out + 1) = r5900AND  (rS1, rT0, rS0);					// EAX &= 0xFFFFFF00
	*(out + 2) = r5900ORI  (rT0, rS1, val);					// EAX |= val

	*skip += 3;
	return 2;
}


int x86MOV_CLIb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	uchar val;

	// Get immediate byte value.
	val = *(in + 1);

	// Encode it...
	* out      = r5900ADDIU(rS0, rR0, 0xFF00);				// S0   = 0xFFFFFF00
	*(out + 1) = r5900AND  (rS1, rT2, rS0);					// ECX &= 0xFFFFFF00
	*(out + 2) = r5900ORI  (rT2, rS1, val);					// ECX |= val

	*skip += 3;
	return 2;
}


int x86MOV_DLIb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	uchar val;

	// Get immediate byte value.
	val = *(in + 1);

	// Encode it...
	* out      = r5900ADDIU(rS0, rR0, 0xFF00);				// S0   = 0xFFFFFF00
	*(out + 1) = r5900AND  (rS1, rT3, rS0);					// EDX &= 0xFFFFFF00
	*(out + 2) = r5900ORI  (rT3, rS1, val);					// EDX |= val

	*skip += 3;
	return 2;
}


int x86MOV_BLIb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	uchar val;

	// Get immediate byte value.
	val = *(in + 1);

	// Encode it...
	* out      = r5900ADDIU(rS0, rR0, 0xFF00);				// S0   = 0xFFFFFF00
	*(out + 1) = r5900AND  (rS1, rT1, rS0);					// EBX &= 0xFFFFFF00
	*(out + 2) = r5900ORI  (rT1, rS1, val);					// EBX |= val

	*skip += 3;
	return 2;
}


int x86MOV_AHIb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	uchar val;

	// Get immediate byte value.
	val = *(in + 1);

	// Encode it...
	* out      = r5900LUI(rS0, 0xFFFF);						// S0   = 0xFFFF0000
	*(out + 1) = r5900ORI(rS1, rS0, 0x00FF);				// S1   = 0xFFFF00FF
	*(out + 2) = r5900AND(rS0, rT0, rS1);					// EAX &= 0xFFFF00FF
	*(out + 3) = r5900ORI(rT0, rS0, val << 8);				// EAX |= val << 8

	*skip += 4;
	return 2;
}


int x86MOV_CHIb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	uchar val;

	// Get immediate byte value.
	val = *(in + 1);

	// Encode it...
	* out      = r5900LUI(rS0, 0xFFFF);						// S0   = 0xFFFF0000
	*(out + 1) = r5900ORI(rS1, rS0, 0x00FF);				// S1   = 0xFFFF00FF
	*(out + 2) = r5900AND(rS0, rT2, rS1);					// ECX &= 0xFFFF00FF
	*(out + 3) = r5900ORI(rT2, rS0, val << 8);				// ECX |= val << 8

	*skip += 4;
	return 2;
}


int x86MOV_DHIb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	uchar val;

	// Get immediate byte value.
	val = *(in + 1);

	// Encode it...
	* out      = r5900LUI(rS0, 0xFFFF);						// S0   = 0xFFFF0000
	*(out + 1) = r5900ORI(rS1, rS0, 0x00FF);				// S1   = 0xFFFF00FF
	*(out + 2) = r5900AND(rS0, rT3, rS1);					// EDX &= 0xFFFF00FF
	*(out + 3) = r5900ORI(rT3, rS0, val << 8);				// EDX |= val << 8

	*skip += 4;
	return 2;
}


int x86MOV_BHIb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	uchar val;

	// Get immediate byte value.
	val = *(in + 1);

	// Encode it...
	* out      = r5900LUI(rS0, 0xFFFF);						// S0   = 0xFFFF0000
	*(out + 1) = r5900ORI(rS1, rS0, 0x00FF);				// S1   = 0xFFFF00FF
	*(out + 2) = r5900AND(rS0, rT1, rS1);					// EBX &= 0xFFFF00FF
	*(out + 3) = r5900ORI(rT1, rS0, val << 8);				// EBX |= val << 8

	*skip += 4;
	return 2;
}


int x86MOV_eAXIv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	ushort sval;

	if (gIsOpSize32)
	{
		// Get upper immediate word value.
		sval = MAKE_WORD(*(in + 4), *(in + 3));

		// Encode it...
		*out = r5900LUI(rT0, sval);							// EAX = 0xuuuu0000

		// Get lower immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// And encode that...
		*(out + 1) = r5900ORI(rT0, rT0, sval);				// EAX = 0xuuuullll

		*skip += 2;
		return 5;
	}
	else
	{
		// Get immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// Encode it...
		* out      = r5900LUI(rS0, 0xFFFF);					// S0   = 0xFFFF0000
		*(out + 1) = r5900AND(rS1, rT0, rS0);				// EAX &= 0xFFFF0000
		*(out + 2) = r5900ORI(rT0, rS1, sval);				// EAX |= 0x0000llll

		*skip += 3;
		return 3;
	}
}


int x86MOV_eCXIv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	ushort sval;

	if (gIsOpSize32)
	{
		// Get upper immediate word value.
		sval = MAKE_WORD(*(in + 4), *(in + 3));

		// Encode it...
		*out = r5900LUI(rT2, sval);							// ECX = 0xuuuu0000

		// Get lower immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// And encode that...
		*(out + 1) = r5900ORI(rT2, rT2, sval);				// ECX = 0xuuuullll

		*skip += 2;
		return 5;
	}
	else
	{
		// Get immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// Encode it...
		* out      = r5900LUI(rS0, 0xFFFF);					// S0   = 0xFFFF0000
		*(out + 1) = r5900AND(rS1, rT2, rS0);				// ECX &= 0xFFFF0000
		*(out + 2) = r5900ORI(rT2, rS1, sval);				// ECX |= 0x0000llll

		*skip += 3;
		return 3;
	}
}


int x86MOV_eDXIv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	ushort sval;

	if (gIsOpSize32)
	{
		// Get upper immediate word value.
		sval = MAKE_WORD(*(in + 4), *(in + 3));

		// Encode it...
		*out = r5900LUI(rT3, sval);							// EDX = 0xuuuu0000

		// Get lower immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// And encode that...
		*(out + 1) = r5900ORI(rT3, rT3, sval);				// EDX = 0xuuuullll

		*skip += 2;
		return 5;
	}
	else
	{
		// Get immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// Encode it...
		* out      = r5900LUI(rS0, 0xFFFF);					// S0   = 0xFFFF0000
		*(out + 1) = r5900AND(rS1, rT3, rS0);				// EDX &= 0xFFFF0000
		*(out + 2) = r5900ORI(rT3, rS1, sval);				// EDX |= 0x0000llll

		*skip += 3;
		return 3;
	}
}


int x86MOV_eBXIv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	ushort sval;

	if (gIsOpSize32)
	{
		// Get upper immediate word value.
		sval = MAKE_WORD(*(in + 4), *(in + 3));

		// Encode it...
		*out = r5900LUI(rT1, sval);							// EBX = 0xuuuu0000

		// Get lower immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// And encode that...
		*(out + 1) = r5900ORI(rT1, rT1, sval);				// EBX = 0xuuuullll

		*skip += 2;
		return 5;
	}
	else
	{
		// Get immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// Encode it...
		* out      = r5900LUI(rS0, 0xFFFF);					// S0   = 0xFFFF0000
		*(out + 1) = r5900AND(rS1, rT1, rS0);				// EBX &= 0xFFFF0000
		*(out + 2) = r5900ORI(rT1, rS1, sval);				// EBX |= 0x0000llll

		*skip += 3;
		return 3;
	}
}


int x86MOV_eSPIv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	ushort sval;

	if (gIsOpSize32)
	{
		// Get upper immediate word value.
		sval = MAKE_WORD(*(in + 4), *(in + 3));

		// Encode it...
		*out = r5900LUI(rT7, sval);							// ESP = 0xuuuu0000

		// Get lower immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// And encode that...
		*(out + 1) = r5900ORI(rT7, rT7, sval);				// ESP = 0xuuuullll

		*skip += 2;
		return 5;
	}
	else
	{
		// Get immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// Encode it...
		* out      = r5900LUI(rS0, 0xFFFF);					// S0   = 0xFFFF0000
		*(out + 1) = r5900AND(rS1, rT7, rS0);				// ESP &= 0xFFFF0000
		*(out + 2) = r5900ORI(rT7, rS1, sval);				// ESP |= 0x0000llll

		*skip += 3;
		return 3;
	}
}


int x86MOV_eBPIv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	ushort sval;

	if (gIsOpSize32)
	{
		// Get upper immediate word value.
		sval = MAKE_WORD(*(in + 4), *(in + 3));

		// Encode it...
		*out = r5900LUI(rT6, sval);							// EBP = 0xuuuu0000

		// Get lower immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// And encode that...
		*(out + 1) = r5900ORI(rT6, rT6, sval);				// EBP = 0xuuuullll

		*skip += 2;
		return 5;
	}
	else
	{
		// Get immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// Encode it...
		* out      = r5900LUI(rS0, 0xFFFF);					// S0   = 0xFFFF0000
		*(out + 1) = r5900AND(rS1, rT6, rS0);				// EBP &= 0xFFFF0000
		*(out + 2) = r5900ORI(rT6, rS1, sval);				// EBP |= 0x0000llll

		*skip += 3;
		return 3;
	}
}


int x86MOV_eSIIv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	ushort sval;

	if (gIsOpSize32)
	{
		// Get upper immediate word value.
		sval = MAKE_WORD(*(in + 4), *(in + 3));

		// Encode it...
		*out = r5900LUI(rT4, sval);							// ESI = 0xuuuu0000

		// Get lower immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// And encode that...
		*(out + 1) = r5900ORI(rT4, rT4, sval);				// ESI = 0xuuuullll

		*skip += 2;
		return 5;
	}
	else
	{
		// Get immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// Encode it...
		* out      = r5900LUI(rS0, 0xFFFF);					// S0   = 0xFFFF0000
		*(out + 1) = r5900AND(rS1, rT4, rS0);				// ESI &= 0xFFFF0000
		*(out + 2) = r5900ORI(rT4, rS1, sval);				// ESI |= 0x0000llll

		*skip += 3;
		return 3;
	}
}


int x86MOV_eDIIv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	ushort sval;

	if (gIsOpSize32)
	{
		// Get upper immediate word value.
		sval = MAKE_WORD(*(in + 4), *(in + 3));

		// Encode it...
		*out = r5900LUI(rT5, sval);							// EDI = 0xuuuu0000

		// Get lower immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// And encode that...
		*(out + 1) = r5900ORI(rT5, rT5, sval);				// EDI = 0xuuuullll

		*skip += 2;
		return 5;
	}
	else
	{
		// Get immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// Encode it...
		* out      = r5900LUI(rS0, 0xFFFF);					// S0   = 0xFFFF0000
		*(out + 1) = r5900AND(rS1, rT5, rS0);				// EDI &= 0xFFFF0000
		*(out + 2) = r5900ORI(rT5, rS1, sval);				// EDI |= 0x0000llll

		*skip += 3;
		return 3;
	}
}


int x86MOV_EbIb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86MOV_EvIv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}
